import type { TRPCClientErrorLike, CreateTRPCProxyClient } from "@trpc/client";
import type { AnyRouter } from "@trpc/server";
import { createQueries, type CreateQueryOptions, type CreateMutationOptions, type CreateInfiniteQueryOptions, type InvalidateQueryFilters, type FetchQueryOptions, type FetchInfiniteQueryOptions, type InfiniteData, type RefetchQueryFilters, type RefetchOptions, type ResetOptions, type CancelOptions, type Updater, type SetDataOptions, type QueryClient, type InvalidateOptions, type QueryFilters, type CreateQueryResult, type CreateInfiniteQueryResult, type CreateMutationResult } from "@tanstack/svelte-query";
type WithNevers<T, V> = {
    [K in keyof T]: Exclude<T[K], undefined> extends V ? never : T[K] extends Record<string, unknown> ? Without<T[K], V> : T[K];
};
type Without<T, V, I = WithNevers<T, V>> = Pick<I, {
    [K in keyof I]: I[K] extends never ? never : K;
}[keyof I]>;
type HasQuery = {
    query: (...args: any[]) => any;
};
type HasMutate = {
    mutate: (...args: any[]) => any;
};
type HasSubscribe = {
    subscribe: (...args: any[]) => any;
};
type OnlyQueries<TClient> = Without<TClient, HasMutate | HasSubscribe>;
declare const UtilsProcedureNames: {
    readonly client: "client";
    readonly fetch: "fetch";
    readonly prefetch: "prefetch";
    readonly fetchInfinite: "fetchInfinite";
    readonly prefetchInfinite: "prefetchInfinite";
    readonly ensureData: "ensureData";
    readonly invalidate: "invalidate";
    readonly refetch: "refetch";
    readonly reset: "reset";
    readonly cancel: "cancel";
    readonly setData: "setData";
    readonly getData: "getData";
    readonly setInfiniteData: "setInfiniteData";
    readonly getInfiniteData: "getInfiniteData";
};
type UtilsProcedures<TInput = undefined, TOutput = undefined, TError = undefined> = {
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientfetchquery
     */
    [UtilsProcedureNames.fetch](input: TInput, opts?: FetchQueryOptions<TInput, TError, TOutput>): Promise<TOutput>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientfetchinfinitequery
     */
    [UtilsProcedureNames.fetchInfinite](input: TInput, opts?: FetchInfiniteQueryOptions<TInput, TError, TOutput>): Promise<InfiniteData<TOutput>>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientprefetchquery
     */
    [UtilsProcedureNames.prefetch](input: TInput, opts?: FetchQueryOptions<TInput, TError, TOutput>): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientprefetchinfinitequery
     */
    [UtilsProcedureNames.prefetchInfinite](input: TInput, opts?: FetchInfiniteQueryOptions<TInput, TError, TOutput>): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/react/reference/QueryClient#queryclientensurequerydata
     */
    [UtilsProcedureNames.ensureData](input?: TInput, opts?: FetchQueryOptions<TInput, TError, TOutput>): Promise<TOutput>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientinvalidatequeries
     */
    [UtilsProcedureNames.invalidate](input?: TInput, filters?: InvalidateQueryFilters, options?: InvalidateOptions): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientrefetchqueries
     */
    [UtilsProcedureNames.refetch](input?: TInput, filters?: RefetchQueryFilters, options?: RefetchOptions): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientcancelqueries
     */
    [UtilsProcedureNames.cancel](input?: TInput, filters?: QueryFilters, options?: CancelOptions): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientresetqueries
     */
    [UtilsProcedureNames.reset](input?: TInput, filters?: QueryFilters, options?: ResetOptions): Promise<void>;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientsetquerydata
     */
    [UtilsProcedureNames.setData](input: TInput, updater: Updater<TOutput | undefined, TOutput | undefined>, options?: SetDataOptions): void;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientsetquerydata
     */
    [UtilsProcedureNames.setInfiniteData](input: TInput, updater: Updater<InfiniteData<TOutput> | undefined, InfiniteData<TOutput> | undefined>, options?: SetDataOptions): void;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientgetquerydata
     */
    [UtilsProcedureNames.getData](input?: TInput): TOutput | undefined;
    /**
     * @link https://tanstack.com/query/v4/docs/reference/QueryClient#queryclientgetquerydata
     */
    [UtilsProcedureNames.getInfiniteData](input?: TInput): InfiniteData<TOutput> | undefined;
};
type AddUtilsPropTypes<TClient, TError> = {
    [K in keyof TClient]: TClient[K] extends HasQuery ? UtilsProcedures<Parameters<TClient[K]["query"]>[0], Awaited<ReturnType<TClient[K]["query"]>>, TError> : AddUtilsPropTypes<TClient[K], TError> & Pick<UtilsProcedures, typeof UtilsProcedureNames.invalidate>;
};
type CreateUtils<TClient, TError> = AddUtilsPropTypes<OnlyQueries<TClient>, TError> & Pick<UtilsProcedures, typeof UtilsProcedureNames.invalidate> & {
    [UtilsProcedureNames.client]: TClient;
};
type CreateQueriesResult<TOpts extends any[]> = ReturnType<typeof createQueries<TOpts>>;
type CreateQueryOptionsForCreateQueries<TOutput, TError, TData> = Omit<CreateQueryOptions<TOutput, TError, TData>, "context" | "queryKey" | "queryFn">;
type CreateQueriesRecord<TClient, TError> = {
    [K in keyof TClient]: TClient[K] extends HasQuery ? <TOutput = Awaited<ReturnType<TClient[K]["query"]>>, TData = TOutput>(input: Parameters<TClient[K]["query"]>[0], opts?: CreateQueryOptionsForCreateQueries<TOutput, TError, TData>) => CreateQueryOptionsForCreateQueries<TOutput, TError, TData> : CreateQueriesRecord<TClient[K], TError>;
};
type CreateQueries<TClient, TError> = <TOpts extends CreateQueryOptionsForCreateQueries<any, any, any>[]>(queriesCallback: (t: CreateQueriesRecord<OnlyQueries<TClient>, TError>) => readonly [...TOpts]) => CreateQueriesResult<TOpts>;
type CreateQueryOptionsForCreateServerQueries<TOutput, TError, TData> = CreateQueryOptionsForCreateQueries<TOutput, TError, TData> & {
    ssr?: boolean;
};
type CreateServerQueriesRecord<TClient, TError> = {
    [K in keyof TClient]: TClient[K] extends HasQuery ? <TOutput = Awaited<ReturnType<TClient[K]["query"]>>, TData = TOutput>(input: Parameters<TClient[K]["query"]>[0], opts?: CreateQueryOptionsForCreateServerQueries<TOutput, TError, TData>) => CreateQueryOptionsForCreateServerQueries<TOutput, TError, TData> : CreateQueriesRecord<TClient[K], TError>;
};
type CreateServerQueries<TClient, TError> = <TOpts extends CreateQueryOptionsForCreateQueries<any, any, any>[]>(queriesCallback: (t: CreateServerQueriesRecord<OnlyQueries<TClient>, TError>) => readonly [...TOpts]) => Promise<(queriesCallback?: (t: CreateQueriesRecord<OnlyQueries<TClient>, TError>, old: readonly [...TOpts]) => readonly [...TOpts]) => CreateQueriesResult<TOpts>>;
declare const ProcedureNames: {
    readonly query: "createQuery";
    readonly serverQuery: "createServerQuery";
    readonly infiniteQuery: "createInfiniteQuery";
    readonly serverInfiniteQuery: "createServerInfiniteQuery";
    readonly mutate: "createMutation";
    readonly subscribe: "createSubscription";
    readonly queryKey: "getQueryKey";
    readonly context: "createContext";
    readonly utils: "createUtils";
    readonly queries: "createQueries";
    readonly serverQueries: "createServerQueries";
};
type CreateTRPCQueryOptions<TOutput, TError, TData> = Omit<CreateQueryOptions<TOutput, TError, TData>, "queryKey" | "queryFn">;
type CreateTRPCServerQueryOptions<TOutput, TError, TData> = CreateTRPCQueryOptions<TOutput, TError, TData> & {
    ssr?: boolean;
};
type TRPCQueryOpts = {
    trpc?: {
        abortOnUnmount?: boolean;
    };
};
type CreateQueryProcedure<TInput, TOutput, TError> = {
    [ProcedureNames.query]: <TData = TOutput>(input: TInput, opts?: CreateTRPCQueryOptions<TOutput, TError, TData> & TRPCQueryOpts) => CreateQueryResult<TData, TError>;
    [ProcedureNames.serverQuery]: <TData = TOutput>(input: TInput, opts?: CreateTRPCServerQueryOptions<TOutput, TError, TData> & TRPCQueryOpts) => Promise<(...args: [TInput | ((old: TInput) => TInput)] | []) => CreateQueryResult<TData, TError>>;
} & {};
type CreateTRPCInfiniteQueryOptions<TOutput, TError, TData> = Omit<CreateInfiniteQueryOptions<TOutput, TError, TData>, "queryKey" | "queryFn">;
type CreateTRPCServerInfiniteQueryOptions<TOutput, TError, TData> = CreateTRPCInfiniteQueryOptions<TOutput, TError, TData> & {
    ssr?: boolean;
};
export type ExtractCursorType<TInput> = TInput extends {
    cursor: any;
} ? TInput["cursor"] : unknown;
type InfiniteQueryOpts<TInput> = {
    initialCursor?: ExtractCursorType<TInput>;
};
type CreateInfiniteQueryProcedure<TInput, TOutput, TError> = (TInput extends {
    cursor?: any;
} ? {
    [ProcedureNames.infiniteQuery]: <TData = TOutput>(input: Omit<TInput, "cursor">, opts?: CreateTRPCInfiniteQueryOptions<TOutput, TError, TData> & InfiniteQueryOpts<TInput> & TRPCQueryOpts) => CreateInfiniteQueryResult<InfiniteData<TData>, TError>;
    [ProcedureNames.serverInfiniteQuery]: <TData = TOutput>(input: Omit<TInput, "cursor">, opts?: CreateTRPCServerInfiniteQueryOptions<TOutput, TError, TData> & InfiniteQueryOpts<TInput> & TRPCQueryOpts) => Promise<(...args: [TInput | ((old: TInput) => TInput)] | []) => CreateInfiniteQueryResult<InfiniteData<TData>, TError>>;
} : {}) & {};
type QueryProcedures<TInput, TOutput, TError> = GetQueryKey<TInput> & CreateQueryProcedure<TInput, TOutput, TError> & CreateInfiniteQueryProcedure<TInput, TOutput, TError>;
type CreateMutationProcedure<TInput, TOutput, TError, TContext = unknown> = {
    [ProcedureNames.mutate]: (opts?: CreateMutationOptions<TOutput, TError, TInput, TContext>) => CreateMutationResult<TOutput, TError, TInput, TContext>;
} & {};
type CreateSubscriptionOptions<TOutput, TError> = {
    enabled?: boolean;
    onStarted?: () => void;
    onData: (data: TOutput) => void;
    onError?: (err: TError) => void;
};
type GetSubscriptionOutput<TOpts> = TOpts extends unknown & Partial<infer A> ? A extends {
    onData: any;
} ? Parameters<A["onData"]>[0] : never : never;
type CreateSubscriptionProcedure<TInput, TOutput, TError> = {
    [ProcedureNames.subscribe]: (input: TInput, opts?: CreateSubscriptionOptions<TOutput, TError>) => void;
} & {};
type AddQueryPropTypes<TClient, TError> = TClient extends Record<any, any> ? {
    [K in keyof TClient]: TClient[K] extends HasQuery ? QueryProcedures<Parameters<TClient[K]["query"]>[0], Awaited<ReturnType<TClient[K]["query"]>>, TError> & {} : TClient[K] extends HasMutate ? CreateMutationProcedure<Parameters<TClient[K]["mutate"]>[0], Awaited<ReturnType<TClient[K]["mutate"]>>, TError> : TClient[K] extends HasSubscribe ? CreateSubscriptionProcedure<Parameters<TClient[K]["subscribe"]>[0], GetSubscriptionOutput<Parameters<TClient[K]["subscribe"]>[1]>, TError> : GetQueryKey & AddQueryPropTypes<TClient[K], TError>;
} : TClient;
type QueryType = "query" | "infinite" | "any";
type QueryKey = [
    string[],
    {
        input?: unknown;
        type?: Exclude<QueryType, "any">;
    }?
];
type GetQueryKey<TInput = undefined> = [TInput] extends [undefined | void] ? {
    [ProcedureNames.queryKey]: () => QueryKey;
} : {
    /**
     * Method to extract the query key for a procedure
     * @param type - defaults to `any`
     */
    [ProcedureNames.queryKey]: (input: TInput, type?: QueryType) => QueryKey;
} & {};
export declare function svelteQueryWrapper<TRouter extends AnyRouter>({ client, queryClient, abortOnUnmount, }: {
    client: CreateTRPCProxyClient<TRouter>;
    queryClient?: QueryClient;
    abortOnUnmount?: boolean;
}): AddQueryPropTypes<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>> & (AddQueryPropTypes<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>> extends Record<any, any> ? {
    /**
     * @deprecated renamed to `createUtils` and will be removed in a future tRPC version
     *
     * @see https://trpc.io/docs/client/react/useUtils
     */
    createContext(): CreateUtils<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
    /**
     * @see https://trpc.io/docs/client/react/useUtils
     */
    createUtils(): CreateUtils<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
    createQueries: CreateQueries<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
    createServerQueries: CreateServerQueries<CreateTRPCProxyClient<TRouter>, TRPCClientErrorLike<TRouter>>;
} : {});
export {};
